@@ -5,15 +5,23 @@ __email__ = 'info@3liz.org'
 import configparser
 import os
 import pwd
+import shutil
 import stat
 
 from pathlib import Path
 from typing import Dict, Union
 
 from qgis_plugin_manager.definitions import Level, Plugin
-
-from .remote import Remote
-from .utils import DEFAULT_QGIS_VERSION, parse_version, pretty_table
+from qgis_plugin_manager.remote import Remote
+from qgis_plugin_manager.utils import (
+    DEFAULT_QGIS_VERSION,
+    parse_version,
+    pretty_table,
+    restart_qgis_server,
+    similar_names,
+    sources_file,
+    to_bool,
+)
 
 
 class LocalDirectory:
@@ -35,9 +43,9 @@ class LocalDirectory:
 
     def init(self) -> bool:
         """ Init this qgis-plugin-manager by creating the default sources.list."""
-        source_file = self.folder.joinpath('sources.list')
+        source_file = sources_file(self.folder)
         if source_file.exists():
-            print(f"{Level.Warning}sources.list already existing. Quit{Level.End}")
+            print(f"{Level.Alert}{source_file.absolute()} is already existing. Quit{Level.End}")
             return False
 
         if self.qgis_version:
@@ -45,7 +53,7 @@ class LocalDirectory:
             print("Init https://plugins.qgis.org")
         else:
             print(
-                f"{Level.Warning}"
+                f"{Level.Alert}"
                 f"QGIS version is unknown, creating with a default {DEFAULT_QGIS_VERSION}"
                 f"{Level.End}"
             )
@@ -56,6 +64,7 @@ class LocalDirectory:
         with open(source_file, 'w', encoding='utf8') as f:
             f.write(server)
 
+        print(f"{source_file.absolute()} has been written.")
         return True
 
     def plugin_list(self) -> Dict[str, str]:
@@ -134,9 +143,64 @@ class LocalDirectory:
                 plugin_folder, "server") in ('True', 'true', '1', 'yes', True),
             has_processing=self.plugin_metadata(
                 plugin_folder, "hasProcessingProvider") in ('True', 'true', '1', 'yes', True),
+            has_wps=self.plugin_metadata(plugin_folder, "wps") in ('True', 'true', '1', 'yes', True),
         )
         return data
 
+    def plugin_installed_version(self, plugin_name) -> Union[str, None]:
+        """ If a plugin is installed or not. """
+        if self._plugins is None:
+            self.plugin_list()
+
+        for plugin_folder in self.plugin_list():
+            info = self.plugin_info(plugin_folder)
+            if info.name == plugin_name:
+                return info.version
+
+        return None
+
+    def remove(self, plugin_name: str) -> bool:
+        """ Remove a plugin by its human name. """
+        if self._plugins is None:
+            self.plugin_list()
+
+        all_names = []
+
+        for plugin_folder in self.plugin_list():
+            info = self.plugin_info(plugin_folder)
+
+            # We fill all names available
+            all_names.append(info.name)
+
+            if info.name == plugin_name:
+                plugin_path = self.folder.joinpath(plugin_folder)
+                try:
+                    shutil.rmtree(plugin_path)
+                except Exception as e:
+                    print(f"{Level.Critical}Plugin {plugin_name} could not be removed : {str(e)}")
+
+                if not Path(self.folder.joinpath(plugin_folder)).exists():
+                    print(f"{Level.Success}Plugin {plugin_name} removed")
+                    restart_qgis_server()
+                    return True
+                else:
+                    print(
+                        f"{Level.Alert}"
+                        f"Plugin {plugin_name} using folder {plugin_folder} could not be removed "
+                        f"for unknown reason"
+                        f"{Level.End}"
+                    )
+                break
+        print(f"{Level.Alert}Plugin name '{plugin_name}' not found{Level.End}")
+
+        all_names = list(set(all_names))
+        similarity = similar_names(plugin_name.lower(), all_names)
+        if similarity:
+            for plugin in similarity:
+                print(f"Do you mean maybe '{plugin}' ?")
+
+        return False
+
     def print_table(self):  # noqa: C901
         """ Print all plugins installed as a table. """
         if self._plugins is None:
@@ -147,7 +211,8 @@ class LocalDirectory:
         print(f"List all plugins in {self.folder.absolute()}\n")
         headers = [
             'Folder ⬇', 'Name', 'Version', 'Flags', 'QGIS min', 'QGIS max', 'Author',
-            'Folder rights', 'Action ⚠', ]
+            'Folder rights', 'Action ⚠',
+        ]
         headers = [f"  {i}  " for i in headers]
         data = []
 
@@ -171,6 +236,8 @@ class LocalDirectory:
             flags = []
             if info.server:
                 flags.append('Server')
+            if info.has_wps:
+                flags.append('WPS')
             if info.experimental in ('True', 'true', '1', 'yes', True):
                 flags.append('Experimental')
             if info.has_processing in ('True', 'true', '1', 'yes', True):
@@ -194,7 +261,12 @@ class LocalDirectory:
             folder = self.folder.joinpath(folder)
             stat_info = os.stat(folder)
             perms = stat.S_IMODE(os.stat(folder).st_mode)
-            permissions = f"{pwd.getpwuid(stat_info.st_uid)[0]} : {oct(perms)}"
+            user_info = stat_info.st_uid
+            try:
+                user_name = pwd.getpwuid(user_info)[0]
+            except KeyError:
+                user_name = user_info
+            permissions = f"{user_name} : {oct(perms)}"
             plugin_data.append(permissions)
             if permissions not in list_of_owners:
                 list_of_owners.append(permissions)
@@ -224,22 +296,25 @@ class LocalDirectory:
                         extra_info.append(f"QGIS Maximum {info.qgis_maximum_version}")
 
             else:
-                extra_info.append('Unknown version')
+                # "qgis-plugin-manager update" is missing.
+                # We can't determine what to do
+                if not to_bool(os.getenv("QGIS_PLUGIN_MANAGER_SKIP_SOURCES_FILE"), False):
+                    extra_info.append('Remote unknown')
 
-            plugin_data.append(Level.Warning + ';'.join(extra_info) + Level.End)
+            plugin_data.append(Level.Alert + ';'.join(extra_info) + Level.End)
             data.append(plugin_data)
 
         if len(data):
             print(pretty_table(data, headers))
         else:
             print(
-                f"{Level.Warning}No plugin found in the current directory {self.folder.absolute()}{Level.End}"
+                f"{Level.Alert}No plugin found in the current directory {self.folder.absolute()}{Level.End}"
             )
 
         if len(list_of_owners) > 1:
             list_of_owners = [f"'{i}'" for i in list_of_owners]
             print(
-                f"{Level.Warning}"
+                f"{Level.Alert}"
                 f"Different rights have been detected : {','.join(list_of_owners)}"
                 f"{Level.End}. "
                 f"Please check user-rights."
